--[[

Game:	 Farming Simulator 22
Title:	 MapCompanion
Script:  Main
Author:  ThundR
Created: 02.06.2022
Updated: 02.23.2022

Copyright ©2022, ThundR (Todd Hundersmarck)

--]]

-- <<[Declarations]>> -----------------------------------------------------------------------------

---@class THMapCompanion:THCore
THMapCompanion = {}
local THMapCompanion_mt = Class(THMapCompanion, THCore)

-- <<[Local Functions]>> --------------------------------------------------------------------------

--[[ runScript --------------------------------]]
-- * runs script startup code
local function runScript()
	g_thMapCompanion = THMapCompanion:new()

    -- hook DensityMapHeightManager function(s)
    g_thMapCompanion:hookFunction(DensityMapHeightManager, "loadMapData", THMapCompanion.DensityMapHeightManager)
end

-- <<[Initialization]>> ---------------------------------------------------------------------------

--[[ new --------------------------------------]]
-- * creates and returns a new environment
    ---@return THMapCompanion
function THMapCompanion:new()
	---@type THMapCompanion
	local newEnv = THCore:new(THMapCompanion_mt)

    if newEnv ~= nil then
        self.densityMapHeightTypesLoaded = false
    end

    return newEnv
end

-- <<{THMapCompanion.DensityMapHeightManager}>> ###################################################

THMapCompanion.DensityMapHeightManager = {}

-- <<[Function Hooks]>> ---------------------------------------------------------------------------

--[[ hook_loadMapData -------------------------]]
-- * called from DensityMapHeightManager.loadMapData()
    ---@param superFunc function @original function
    ---@param xmlFile number @map xmlFile id
    ---@param missionInfo table @mission information table
    ---@param baseDirectory string @map base directory
function THMapCompanion.DensityMapHeightManager:hook_loadMapData(superFunc, xmlFile, missionInfo, baseDirectory, ...)
    local success, ra,rb,rc,rd,re = superFunc(self, xmlFile, missionInfo, baseDirectory, ...)

    if success and not g_thMapCompanion.densityMapHeightTypesLoaded then
        local modPath = g_thMapCompanion.modPath
        local modDescFilename = g_thMapCompanion.xmlFilename
        local modDescXMLFile = loadXMLFile("modDescXML", modDescFilename)

        local ttgXMLFilename = getXMLString(modDescXMLFile, "modDesc.densityMapHeightTypes#filename")
        if ttgXMLFilename ~= nil then
            ttgXMLFilename = Utils.getFilename(ttgXMLFilename, modPath)
            if fileExists(ttgXMLFilename) then
                local ttgXMLFile = loadXMLFile("ttgXMLFile", ttgXMLFilename)
                if Utils.getNoNil(ttgXMLFile, 0) ~= 0 then
                    self:loadDensityMapHeightTypes(ttgXMLFile, missionInfo, modPath, false)
                end
            end
        end

        g_thMapCompanion.densityMapHeightTypesLoaded = true
    end

    return success, ra,rb,rc,rd,re
end

-- <<{Script Start}>> #############################################################################

runScript()